# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dydxprotocol/clob/order_removals.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from ..clob import order_pb2 as dydxprotocol_dot_clob_dot_order__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&dydxprotocol/clob/order_removals.proto\x12\x11\x64ydxprotocol.clob\x1a\x14gogoproto/gogo.proto\x1a\x1d\x64ydxprotocol/clob/order.proto\"\xe8\x04\n\x0cOrderRemoval\x12\x32\n\x08order_id\x18\x01 \x01(\x0b\x32\x1a.dydxprotocol.clob.OrderIdB\x04\xc8\xde\x1f\x00\x12\x45\n\x0eremoval_reason\x18\x02 \x01(\x0e\x32-.dydxprotocol.clob.OrderRemoval.RemovalReason\"\xdc\x03\n\rRemovalReason\x12\x1e\n\x1aREMOVAL_REASON_UNSPECIFIED\x10\x00\x12&\n\"REMOVAL_REASON_UNDERCOLLATERALIZED\x10\x01\x12&\n\"REMOVAL_REASON_INVALID_REDUCE_ONLY\x10\x02\x12\x34\n0REMOVAL_REASON_POST_ONLY_WOULD_CROSS_MAKER_ORDER\x10\x03\x12%\n!REMOVAL_REASON_INVALID_SELF_TRADE\x10\x04\x12<\n8REMOVAL_REASON_CONDITIONAL_FOK_COULD_NOT_BE_FULLY_FILLED\x10\x05\x12\x35\n1REMOVAL_REASON_CONDITIONAL_IOC_WOULD_REST_ON_BOOK\x10\x06\x12\x1f\n\x1bREMOVAL_REASON_FULLY_FILLED\x10\x07\x12;\n7REMOVAL_REASON_VIOLATES_ISOLATED_SUBACCOUNT_CONSTRAINTS\x10\x08\x12+\n\'REMOVAL_REASON_PERMISSIONED_KEY_EXPIRED\x10\tB8Z6github.com/dydxprotocol/v4-chain/protocol/x/clob/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dydxprotocol.clob.order_removals_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z6github.com/dydxprotocol/v4-chain/protocol/x/clob/types'
  _ORDERREMOVAL.fields_by_name['order_id']._options = None
  _ORDERREMOVAL.fields_by_name['order_id']._serialized_options = b'\310\336\037\000'
  _ORDERREMOVAL._serialized_start=115
  _ORDERREMOVAL._serialized_end=731
  _ORDERREMOVAL_REMOVALREASON._serialized_start=255
  _ORDERREMOVAL_REMOVALREASON._serialized_end=731
# @@protoc_insertion_point(module_scope)
