# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dydxprotocol/accountplus/params.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from v4_proto.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%dydxprotocol/accountplus/params.proto\x12\x18\x64ydxprotocol.accountplus\x1a\x14gogoproto/gogo.proto\"M\n\x06Params\x12\x43\n\x17is_smart_account_active\x18\x01 \x01(\x08\x42\"\xf2\xde\x1f\x1eyaml:\"is_smart_account_active\"B?Z=github.com/dydxprotocol/v4-chain/protocol/x/accountplus/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dydxprotocol.accountplus.params_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z=github.com/dydxprotocol/v4-chain/protocol/x/accountplus/types'
  _PARAMS.fields_by_name['is_smart_account_active']._options = None
  _PARAMS.fields_by_name['is_smart_account_active']._serialized_options = b'\362\336\037\036yaml:\"is_smart_account_active\"'
  _PARAMS._serialized_start=89
  _PARAMS._serialized_end=166
# @@protoc_insertion_point(module_scope)
